/*
 *  Copyright (C) 2010  Regents of the University of Michigan
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//////////////////////////////////////////////////////////////////////////
// This file contains the processing for the executable option "dumpIndex"
// which prints a BAM Index File in a readable format.

#include "SamFile.h"
#include "Parameters.h"

void dumpIndexDescription()
{
    std::cout << " dumpIndex - Print BAM Index File in English:" << std::endl;
    std::cout << "\t./bam dumpIndex <bamIndexFile> <ref#>" << std::endl;
}


void dumpIndexUsage()
{
    dumpIndexDescription();
    std::cout << "\tRequired Parameters:" << std::endl;
    std::cout << "\t\tbamIndexFile - path/name of the index file to display\n";
    std::cout << "\tOptional Parameters:" << std::endl;
    std::cout << "\t\tref# - the reference number to print (optional) defaults to print all\n";
    std::cout << std::endl;
}


// Dump the specified Bam Index file.
int dumpIndex(int argc, char **argv)
{
    const char* filename = argv[2];
    int32_t refID = -1;
    if(argc == 4)
    {
        refID = strtol(argv[3], NULL, 10);
    }

    // Read the index.
    BamIndex bamIndex;
    SamStatus status;
    status = bamIndex.readIndex(filename);

    if(status != SamStatus::SUCCESS)
    {
        // Failed to read the index, return.
        fprintf(stderr, "%s\n", status.getStatusMessage());
        return(status.getStatus());
    }

    bamIndex.printIndex(refID);
    
    return(status.getStatus());
}


